@@ -9,15 +9,17 @@ import shutil
 from contextlib import ExitStack
 from pathlib import Path
 from tempfile import NamedTemporaryFile
-from typing import IO, Dict, Iterator, List, Set, Tuple, Union, cast
+from typing import IO, Dict, Iterator, List, Set, Tuple, cast
 
 from packaging.requirements import Requirement
 from packaging.specifiers import SpecifierSet
 from packaging.version import Version
-from pip_api import Requirement as ParsedRequirement
-from pip_api import parse_requirements
-from pip_api._parse_requirements import UnparsedRequirement
-from pip_api.exceptions import PipError
+from pip_requirements_parser import (
+    EditableRequirement,
+    InstallRequirement,
+    InvalidRequirementLine,
+    RequirementsFile,
+)
 
 from pip_audit._dependency_source import (
     DependencyFixError,
@@ -82,13 +84,40 @@ class RequirementSource(DependencySource):
         collected: Set[Dependency] = set()
         for filename in self._filenames:
             try:
-                reqs = parse_requirements(filename=filename)
-            except PipError as pe:
-                raise RequirementSourceError(
-                    f"requirement parsing raised an error: {filename}"
-                ) from pe
-            try:
-                for _, dep in self._collect_cached_deps(filename, list(reqs.values())):
+                rf = RequirementsFile.from_file(filename)
+                if rf.invalid_lines:
+                    raise RequirementSourceError(
+                        f"requirement file {filename} contains invalid lines: "
+                        f"{str(rf.invalid_lines)}"
+                    )
+
+                reqs: List[InstallRequirement] = []
+                req_names: Set[str] = set()
+                for req in rf.requirements:
+                    if req.req is None:
+                        # For editable requirements that don't have an egg fragment that lists the
+                        # the package name and version, `pip-requirements-parser` won't attach a
+                        # `Requirement` object to the `InstallRequirement`.
+                        #
+                        # In this case, we can't audit the dependency so we should signal to the
+                        # caller that we're skipping it.
+                        assert isinstance(req, EditableRequirement)
+                        yield SkippedDependency(
+                            name=req.requirement_line.line,
+                            skip_reason="could not deduce package/specifier pair from requirement, "
+                            "please specify them with #egg=your_package_name==your_package_version",
+                        )
+                        continue
+                    if req.marker is None or req.marker.evaluate():
+                        # This means we have a duplicate requirement for the same package
+                        if req.name in req_names:
+                            raise RequirementSourceError(
+                                f"package {req.name} has duplicate requirements: {str(req)}"
+                            )
+                        req_names.add(req.name)
+                        reqs.append(req)
+
+                for _, dep in self._collect_cached_deps(filename, reqs):
                     if dep in collected:
                         continue
                     collected.add(dep)
@@ -128,27 +157,44 @@ class RequirementSource(DependencySource):
     def _fix_file(self, filename: Path, fix_version: ResolvedFixVersion) -> None:
         # Reparse the requirements file. We want to rewrite each line to the new requirements file
         # and only modify the lines that we're fixing.
-        try:
-            reqs = parse_requirements(filename=filename)
-        except PipError as pe:
-            raise RequirementFixError(f"requirement parsing raised an error: {filename}") from pe
+        #
+        # This time we're using the `RequirementsFile.parse` API instead of `Requirements.from_file`
+        # since we want to access each line sequentially in order to rewrite the file.
+        reqs = list(RequirementsFile.parse(filename=str(filename)))
 
-        # Convert requirements types from pip-api's vendored types to our own
-        req_list: List[Requirement] = [Requirement(str(req)) for req in reqs.values()]
+        # Check ahead of time for anything invalid in the requirements file since we don't want to
+        # encounter this while writing out the file. Check for duplicate requirements and lines that
+        # failed to parse.
+        req_names: Set[str] = set()
+        for req in reqs:
+            if (
+                isinstance(req, InstallRequirement)
+                and (req.marker is None or req.marker.evaluate())
+                and req.req is not None
+            ):
+                if req.name in req_names:
+                    raise RequirementFixError(
+                        f"package {req.name} has duplicate requirements: {str(req)}"
+                    )
+                req_names.add(req.name)
+            elif isinstance(req, InvalidRequirementLine):
+                raise RequirementFixError(
+                    f"requirement file {filename} has invalid requirement: {str(req)}"
+                )
 
         # Now write out the new requirements file
         with filename.open("w") as f:
             fixed = False
-            for req in req_list:
+            for req in reqs:
                 if (
-                    req.name == fix_version.dep.name
+                    isinstance(req, InstallRequirement)
+                    and req.name == fix_version.dep.name
                     and req.specifier.contains(fix_version.dep.version)
                     and not req.specifier.contains(fix_version.version)
                 ):
-                    req.specifier = SpecifierSet(f"=={fix_version.version}")
+                    req.req.specifier = SpecifierSet(f"=={fix_version.version}")
                     fixed = True
-                assert req.marker is None or req.marker.evaluate()
-                print(str(req), file=f)
+                print(req.dumps(), file=f)
 
             # The vulnerable dependency may not be explicitly listed in the requirements file if it
             # is a subdependency of a requirement. In this case, we should explicitly add the fixed
@@ -159,8 +205,11 @@ class RequirementSource(DependencySource):
             # another.
             try:
                 if not fixed:
+                    installed_reqs: List[InstallRequirement] = [
+                        r for r in reqs if isinstance(r, InstallRequirement)
+                    ]
                     origin_reqs: Set[Requirement] = set()
-                    for req, dep in self._collect_cached_deps(filename, list(reqs.values())):
+                    for req, dep in self._collect_cached_deps(filename, list(installed_reqs)):
                         if fix_version.dep == dep:
                             origin_reqs.add(req)
                     if origin_reqs:
@@ -194,7 +243,7 @@ class RequirementSource(DependencySource):
 
     def _collect_preresolved_deps(
         self,
-        reqs: Iterator[Union[ParsedRequirement, UnparsedRequirement]],
+        reqs: Iterator[InstallRequirement],
         require_hashes: bool = False,
     ) -> Iterator[Tuple[Requirement, Dependency]]:
         """
@@ -202,8 +251,7 @@ class RequirementSource(DependencySource):
         hash requirement policy.
         """
         for req in reqs:
-            req = cast(ParsedRequirement, req)
-            if require_hashes and not req.hashes:
+            if require_hashes and not req.hash_options:
                 raise RequirementSourceError(
                     f"requirement {req.name} does not contain a hash {str(req)}"
                 )
@@ -215,12 +263,27 @@ class RequirementSource(DependencySource):
             if pinned_specifier is None:
                 raise RequirementSourceError(f"requirement {req.name} is not pinned: {str(req)}")
 
-            yield Requirement(str(req)), ResolvedDependency(
-                req.name, Version(pinned_specifier.group("version")), req.hashes
+            yield req.req, ResolvedDependency(
+                req.name,
+                Version(pinned_specifier.group("version")),
+                self._build_hash_options_mapping(req.hash_options),
             )
 
+    def _build_hash_options_mapping(self, hash_options: List[str]) -> Dict[str, List[str]]:
+        """
+        A helper that takes a list of hash options and returns a dictionary mapping from hash
+        algorithm (e.g. sha256) to a list of values.
+        """
+        mapping: Dict[str, List[str]] = {}
+        for hash_option in hash_options:
+            algorithm, hash_ = hash_option.split(":")
+            if algorithm not in mapping:
+                mapping[algorithm] = []
+            mapping[algorithm].append(hash_)
+        return mapping
+
     def _collect_cached_deps(
-        self, filename: Path, reqs: List[Union[ParsedRequirement, UnparsedRequirement]]
+        self, filename: Path, reqs: List[InstallRequirement]
     ) -> Iterator[Tuple[Requirement, Dependency]]:
         """
         Collect resolved dependencies for a given requirements file, retrieving them from the
@@ -241,9 +304,7 @@ class RequirementSource(DependencySource):
         # 2. One or more parsed requirements has hashes specified, enabling
         #    hash checking for all requirements.
         # 3. The user has explicitly specified `--no-deps`.
-        require_hashes = self._require_hashes or any(
-            isinstance(req, ParsedRequirement) and req.hashes for req in reqs
-        )
+        require_hashes = self._require_hashes or any(req.hash_options for req in reqs)
         skip_deps = require_hashes or self._no_deps
         if skip_deps:
             for req, dep in self._collect_preresolved_deps(
@@ -255,7 +316,7 @@ class RequirementSource(DependencySource):
                 yield req, dep
         else:
             # Invoke the dependency resolver to turn requirements into dependencies
-            req_values: List[Requirement] = [Requirement(str(req)) for req in reqs]
+            req_values: List[Requirement] = [r.req for r in reqs]
             for req, resolved_deps in self._resolver.resolve_all(iter(req_values)):
                 for dep in resolved_deps:
                     if req not in new_cached_deps_for_file:
